"""
Teamly API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the Teamly API.

:copyright: (c) 2025 Fatih Kuloglu
:license: MIT, see LICENSE for more details.

"""

__title__ = 'teamly'
__author__ = 'MrFatihLD'
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2025 Fatih Kuloglu'
__version__ = '0.1.0a'

from .client import *
from .http import *
from .utils import *
from .gateway import *
